/* emTest.c
**
** Test program for the `emHeader' structure.
**
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include "prefs.h"
#include "inline-funcs.h"
#include "mrmacs.h"
#include "exitcodes.h"
#include "vxerror.h"
#include "tools.h"

#include "hread.h"
#include "emHeader.h"

static char *xGet (char *line, size_t max, FILE *out, FILE *in, char *prompt)
{
    char rest[1024];
    fprintf (out, "%s: ", prompt); fflush (out);
    ifnull (fgets (line, max, in)) { return NULL; }
    if (cuteol(line)) { return line; }
    while (fgets (rest, 1024, in) && !cuteol (rest));
    fprintf (stderr, "*** line too long ***\n");
    return line;
}

char *pGet (FILE *out, FILE *in, char *prompt)
{
    static char line[1024];
    return xGet (line, 1024, out, in, prompt);
}

int pNum (FILE *out, FILE *in, char *prompt, long *_num)
{
    char line[1024];
    long res;
    char *p, *s = xGet (line, 1024, out, in, prompt);
    ifnull (s) { return -1; }
    while (isws (*s)) { ++s; }
    res = strtol (s, &p, 0);
    if (*p) { fprintf (stderr, "*** invalid number ***\n"); }
    *_num = res;
    return 0;
}

int main (int argc, char *argv[])
{
    FILE *fp;
    int l1 = 1, l2 = 0, ec, head = 1, ac, pos;
    char *li, *p, *q, *line;
    ehList_t eh;
    emHeader_t emh = { (ehList_t) NULL, (ehList_t) NULL };
    progname = strrchr (argv[0], '/');
    ifnull (progname) { progname = argv[0]; } else { ++progname; }
    if (argc < 2) {
	fprintf (stderr, "Usage: %s <file>\n", progname);
	exit (EX_USAGE);
    }
    ifnull (fp = fopen (argv[1], "r")) {
	vxerror (EX_OSERR, errno, "attempt to read `%s' failed", argv[1]);
    }
    for (;;) {
	li = hread_line (fp, head);
	if (!*li) { break; }
	l2 = hread_lineno();
/*	printf ("%.2d-%.2d: %s (%d chars)\n\n", l1, l2, li, strlen (li));*/
	l1 = l2 + 1;
	if (!lccmp (li, "From ", 5)) {
	    ec = emHeader_ins (&emh, "From ", &li[5]);
	} else {
	    ifnull (p = strchr (li, ':')) {
		fprintf (stderr, "##ERROR: invalid header line - ignored!\n");
		continue;
	    }
	    q = p; *p++ = '\0'; --q;
	    if (isws (*p)) { ++p; }
	    while (isws (*li)) { ++li; }
	    while (q - li > 0 && isws (*q)) { --q; }
	    *++q = '\0';
	    ec = emHeader_ins (&emh, li, p);
	}
	if (ec) {
	    fprintf (stderr, "--- double header element `%s'.\n", li);
	}
    }
/*    printf ("End of header found at line %d\n", hread_lineno());*/
    for (eh = emh.f; noNULL(eh); eh = eh->next) {
	printf ("%s : %s\n", eh->name, eh->value);
    }
    for (;;) {
	printf ("\n\nHeader elements:");
	for (eh = emh.f; noNULL(eh); eh = eh->next) {
	    printf (" %s: %s\n", eh->name, eh->value);
	}
	line = pGet (stdout, stdin, "\n\nPlease enter an action");
	ifnull (line) { break; }
	p = line; while (isws(*p)) { ++p; }
	if (!*p) { continue; }
	ac = (int) *p++;
	while (isws(*p)) { ++p; }
	q = p; while (*p && !isws(*p)) { ++p; }
	switch (ac) {
	    case 'a':
		if (q == p) {
		    fprintf (stderr, "a requires two args!\n");
		    continue;
		}
		if (isws(*p)) {
		    *p++ = '\0';
		    while (isws(*p)) { ++p; }
		    if (!*p) {
			fprintf (stderr, "a requires two args!\n");
			continue;
		    }
		}
		emHeader_ins (&emh, q, p);
		break;
	    case 'd':
		if (q == p) {
		    fprintf (stderr, "d requires two args!\n");
		    continue;
		}
		if (isws(*p)) {
		    *p++ = '\0';
		    while (isws(*p)) { ++p; }
		    if (!*p) {
			fprintf (stderr, "a requires two args!\n");
			continue;
		    }
		}
		pos = atoi (p);
		emHeader_del (&emh, q, pos);
		break;
	    case 'r':
		fprintf (stderr, "r is not implemented (yet).\n");
		continue;
	    default:
		fprintf (stderr, "%c is no valid command.\n", ac);
		continue;
	}
    }
    printf ("\n\n### done. ###\n");
    return 0;
}
